package net.avicus.leap;

import java.util.logging.Level;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import net.avicus.api.utils.Config;
import net.avicus.api.utils.Gamer;
import net.avicus.api.utils.Chat;
import net.avicus.leap.api.Trail;
import net.avicus.leap.cmds.TrailCmd;
import net.avicus.leap.listeners.LeapListener;

import org.bukkit.Bukkit;
import org.bukkit.Effect;
import org.bukkit.Sound;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;

public class Leap extends JavaPlugin
{
	// beeselmane --> added getters instead of @Getter (changed modifyer to public)
	private HashMap<Effect, String> effects = new HashMap<Effect, String>();
	private List<Sound> sounds = new ArrayList<Sound>();
	private double velocity;
	private double elevation;

	public HashMap<Effect, String> getEffects()
	{
		return this.effects;
	}

	public List<Sound> getSounds()
	{
		return this.sounds;
	}

	public double getVelocity()
	{
		return this.velocity;
	}

	public double getElevation()
	{
		return this.elevation;
	}
	// beeselmane --> end block

	private boolean anticheat;
	
	private HashMap<String, Trail> trails = new HashMap<String, Trail>();

	@Override
	public void onEnable()
	{
		new net.avicus.api.events.LeapEventFactory(this);
		Chat.Init(this);
		net.avicus.api.events.Schedule.Init(this); // beeselmane --> init schedular (the way it's coded, it's probably in their external dependant plugin on their server)
		Config config = new Config(this);
		
		for (String raw : config.getStringList("effects"))
		{
			try {
				String[] parts = raw.split(":");
				
				Effect effect = Effect.valueOf(parts[0].toUpperCase().replace(" ", "_"));
				String perm = parts.length == 1 ? null : parts[1];
				
				effects.put(effect, perm);
			} catch (Exception e) {
				Chat.log(Level.WARNING, "Invalid effect name: \"%s\"", raw);
			}
		}
		
		for (String raw : config.getStringList("sounds")) {
			try {
				Sound sound = Sound.valueOf(raw.toUpperCase().replace(" ", "_"));
				sounds.add(sound);
			} catch (Exception e) {
				Chat.log(Level.WARNING, "Invalid effect name: \"%s\"", raw);
			}
		}
		
		for (String input : config.getStringList("trails")) {
			Trail trail = Trail.getByName(input);
			if (trail != null)
				trail.setEnabled(true);
		}
		
		velocity = config.getDouble("velocity");
		elevation = config.getDouble("elevation");
		
		anticheat = Bukkit.getServer().getPluginManager().getPlugin("AntiCheat") != null;
		
		if (anticheat) Chat.log(Level.INFO, "Connected to AntiCheat!");
		
		Bukkit.getPluginManager().registerEvents(new LeapListener(this), this);
		getCommand("trail").setExecutor(new TrailCmd(this));
	}
	
	/**
	 * @param player
	 * @param effect
	 * @return If the player has permission to use the effect
	 */
	public boolean hasEffect(Player player, Effect effect) {
		String perm = effects.get(effect);
		
		if (perm == null)
			return true;
		
		return player.hasPermission(perm);
	}
	
	/**
	 * @return If the current server is running the AntiCheat plugin
	 */
	public boolean hasAntiCheat() {
		return anticheat;
	}
	
	/**
	 * @param gamer
	 * @return List of trails that this gamer has permission to use
	 */
	public List<Trail> getTrails(Gamer gamer) {
		List<Trail> trails = new ArrayList<Trail>();
		
		for (Trail trail : Trail.getList()) {
			if (trail.isEnabled() == false)
				continue;
			
			// if (trail.hasPermission(gamer))
				trails.add(trail);
		}
		
		return trails;
	}
	
	/**
	 * @param username
	 * @return The trail that the specified user is currently using. Returns null if none.
	 */
	public Trail getTrail(String username)
	{
		if (trails.containsKey(username) == false) return null;
		return trails.get(username);
	}
	
	/**
	 * Set the trail of a gamer
	 * @param g The gamer
	 * @param trail The trail to set
	 */
	public void setTrail(Gamer g, Trail trail)
	{
		trails.put(g.getName(), trail);
	}
	
}
